"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Sidecar = void 0;
const config_js_1 = require("../../config.js");
const sidecar_body_js_1 = require("../../sidecar-body/sidecar-body.js");
const build_sidecar_metadata_js_1 = require("./build-sidecar-metadata.js");
const guard_metadata_sidecar_js_1 = require("./guard-metadata-sidecar.js");
const metadata_sidecar_js_1 = require("./metadata-sidecar.js");
function Sidecar(sidecarTarget, initAgentScript) {
    config_js_1.log.verbose('Sidecar', '@Sidecar(%s%s)', Array.isArray(sidecarTarget)
        ? JSON.stringify(sidecarTarget)
        : sidecarTarget, initAgentScript
        ? `, "${initAgentScript.substr(0, 20)}..."`
        : '');
    return classDecorator;
    /**
     * See: https://www.typescriptlang.org/docs/handbook/decorators.html#class-decorators
     */
    function classDecorator(Klass) {
        config_js_1.log.verbose('Sidecar', '@Sidecar(%s%s) classDecorator(%s)', Array.isArray(sidecarTarget)
            ? JSON.stringify(sidecarTarget)
            : (sidecarTarget || ''), `"${initAgentScript?.substr(0, 20)}..."` || '', Klass.name);
        // https://stackoverflow.com/a/14486171/1123955
        if (!(Klass.prototype instanceof sidecar_body_js_1.SidecarBody)) {
            throw new Error('Sidecar: the class decorated by @Sidecar must extends from `SidecarBody`');
        }
        const meta = (0, build_sidecar_metadata_js_1.buildSidecarMetadata)(Klass, {
            initAgentScript,
            sidecarTarget,
        });
        /**
         * Validate metadata for sidecar
         */
        (0, guard_metadata_sidecar_js_1.guardMetadataSidecar)(meta);
        /**
         * Save metadata
         */
        (0, metadata_sidecar_js_1.updateMetadataSidecar)(Klass, meta);
    }
}
exports.Sidecar = Sidecar;
//# sourceMappingURL=sidecar.js.map