#!/usr/bin/env -S node --no-warnings --loader ts-node/esm
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tstest_1 = require("tstest");
const function_target_js_1 = require("./function-target.js");
(0, tstest_1.test)('addressTarget()', async (t) => {
    const DATA = 0x1234;
    const EXPECTED = {
        address: '0x1234',
        moduleName: null,
        type: 'address',
    };
    const result = (0, function_target_js_1.addressTarget)(DATA);
    t.same(result, EXPECTED, 'should get the correct address target for number');
});
(0, tstest_1.test)('addressTarget() with module', async (t) => {
    const DATA = 0x1234;
    const MODULE_NAME = 'myModule';
    const EXPECTED = {
        address: '0x1234',
        moduleName: MODULE_NAME,
        type: 'address',
    };
    const result = (0, function_target_js_1.addressTarget)(DATA, MODULE_NAME);
    t.same(result, EXPECTED, 'should get the correct address target for number and module name');
});
(0, tstest_1.test)('agentTarget()', async (t) => {
    const DATA = 'myPtr';
    const EXPECTED = {
        funcName: DATA,
        type: 'agent',
    };
    const result = (0, function_target_js_1.agentTarget)(DATA);
    t.same(result, EXPECTED, 'should get the correct agent target for var name');
});
(0, tstest_1.test)('exportTarget()', async (t) => {
    const DATA = 'testExport';
    const EXPECTED = {
        exportName: DATA,
        moduleName: null,
        type: 'export',
    };
    const result = (0, function_target_js_1.exportTarget)(DATA);
    t.same(result, EXPECTED, 'should get the correct export target for number');
});
(0, tstest_1.test)('exportTarget() with export', async (t) => {
    const DATA = 'testExport';
    const MODULE_NAME = 'myModule';
    const EXPECTED = {
        exportName: DATA,
        moduleName: MODULE_NAME,
        type: 'export',
    };
    const result = (0, function_target_js_1.exportTarget)(DATA, MODULE_NAME);
    t.same(result, EXPECTED, 'should get the correct export target for name and module name');
});
//# sourceMappingURL=function-target.spec.js.map